@@ -1,91 +1,16 @@
 from django.conf import settings
 from django.db import models
 from django.utils.safestring import mark_safe
-from django.utils.html import linebreaks, urlize
-from django.utils.functional import curry
-import widgets
+from django.utils.html import escape
+
+from markupfield import widgets
+from markupfield import markup
 
 _rendered_field_name = lambda name: '_%s_rendered' % name
 _markup_type_field_name = lambda name: '%s_markup_type' % name
 
-# build _DEFAULT_MARKUP_TYPES
-_DEFAULT_MARKUP_TYPES = {
-    'html': lambda markup: markup,
-    'plain': lambda markup: urlize(linebreaks(markup)),
-}
-
-try:
-    import pygments
-    PYGMENTS_INSTALLED = True
-
-    def _register_pygments_rst_directive():
-        from docutils import nodes
-        from docutils.parsers.rst import directives
-        from pygments import highlight
-        from pygments.lexers import get_lexer_by_name, TextLexer
-        from pygments.formatters import HtmlFormatter
-
-        DEFAULT = HtmlFormatter()
-        VARIANTS = {
-            'linenos': HtmlFormatter(linenos=True),
-        }
-
-        def pygments_directive(name, arguments, options, content, lineno,
-                               content_offset, block_text, state, state_machine):
-            try:
-                lexer = get_lexer_by_name(arguments[0])
-            except ValueError:
-                # no lexer found - use the text one instead of an exception
-                lexer = TextLexer()
-            formatter = options and VARIANTS[options.keys()[0]] or DEFAULT
-            parsed = highlight(u'\n'.join(content), lexer, formatter)
-            return [nodes.raw('', parsed, format='html')]
-        pygments_directive.arguments = (1, 0, 1)
-        pygments_directive.content = 1
-        directives.register_directive('code', pygments_directive)
-
-except ImportError:
-    PYGMENTS_INSTALLED = False
-
-try:
-    import markdown
-    _DEFAULT_MARKUP_TYPES['markdown'] = markdown.markdown
-
-    # try and replace if pygments & codehilite are available
-    if PYGMENTS_INSTALLED:
-        try:
-            from markdown.extensions.codehilite import makeExtension
-            _DEFAULT_MARKUP_TYPES['markdown'] = curry(markdown.markdown, extensions=['codehilite(css_class=highlight)'])
-        except ImportError:
-            pass
-
-except ImportError:
-    pass
-
-try:
-    from docutils.core import publish_parts
-
-    if PYGMENTS_INSTALLED:
-        _register_pygments_rst_directive()
-
-    def render_rest(markup):
-        overrides = getattr(settings, "RESTRUCTUREDTEXT_FILTER_SETTINGS", {})
-        parts = publish_parts(source=markup, writer_name="html4css1",
-                              settings_overrides=overrides)
-        return parts["fragment"]
-
-    _DEFAULT_MARKUP_TYPES['restructuredtext'] = render_rest
-except ImportError:
-    pass
-
-try:
-    import textile
-    _DEFAULT_MARKUP_TYPES['textile'] = curry(textile.textile,
-                                             encoding='utf-8', output='utf-8')
-except ImportError:
-    pass
-
-_MARKUP_TYPES = getattr(settings, 'MARKUP_FIELD_TYPES', _DEFAULT_MARKUP_TYPES)
+# for fields that don't set markup_types: detected types or from settings
+_MARKUP_TYPES = getattr(settings, 'MARKUP_FIELD_TYPES', markup.DEFAULT_MARKUP_TYPES)
 
 
 class Markup(object):
@@ -155,22 +80,43 @@ class MarkupDescriptor(object):
 class MarkupField(models.TextField):
 
     def __init__(self, verbose_name=None, name=None, markup_type=None,
-                 default_markup_type=None, **kwargs):
+                 default_markup_type=None, markup_choices=_MARKUP_TYPES,
+                 escape_html=False, **kwargs):
+
         if markup_type and default_markup_type:
             raise ValueError('Cannot specify both markup_type and default_markup_type')
+
         self.default_markup_type = markup_type or default_markup_type
-        if (self.default_markup_type and
-            self.default_markup_type not in _MARKUP_TYPES):
-            raise ValueError("Invalid markup type for field '%s', allowed values: %s" %
-                             (name, ', '.join(_MARKUP_TYPES.iterkeys())))
         self.markup_type_editable = markup_type is None
+        self.escape_html = escape_html
+
+        # pre 1.0 markup_choices might have been a dict
+        if isinstance(markup_choices, dict):
+            raise DeprecationWarning('passing a dictionary as markup_choices is deprecated')
+            self.markup_choices_dict = markup_choices
+            self.markup_choices_list = markup_choices.keys()
+        else:
+            self.markup_choices_list = [mc[0] for mc in markup_choices]
+            self.markup_choices_dict = dict(markup_choices)
+
+        if (self.default_markup_type and
+            self.default_markup_type not in self.markup_choices_list):
+            raise ValueError("Invalid default_markup_type for field '%s', allowed values: %s" %
+                             (name, ', '.join(self.markup_choices_list)))
+
+        # for South FakeORM compatibility: the frozen version of a
+        # MarkupField can't try to add a _rendered field, because the
+        # _rendered field itself is frozen as well. See introspection
+        # rules below.
+        self.rendered_field = not kwargs.pop('rendered_field', False)
+
         super(MarkupField, self).__init__(verbose_name, name, **kwargs)
 
     def contribute_to_class(self, cls, name):
         if not cls._meta.abstract:
-            keys = _MARKUP_TYPES.keys()
+            choices = zip(self.markup_choices_list, self.markup_choices_list)
             markup_type_field = models.CharField(max_length=30,
-                choices=zip(keys, keys), default=self.default_markup_type,
+                choices=choices, default=self.default_markup_type,
                 editable=self.markup_type_editable, blank=self.blank)
             rendered_field = models.TextField(editable=False)
             markup_type_field.creation_counter = self.creation_counter+1
@@ -183,11 +129,15 @@ class MarkupField(models.TextField):
 
     def pre_save(self, model_instance, add):
         value = super(MarkupField, self).pre_save(model_instance, add)
-        if value.markup_type not in _MARKUP_TYPES:
+        if value.markup_type not in self.markup_choices_list:
             raise ValueError('Invalid markup type (%s), allowed values: %s' %
                              (value.markup_type,
-                              ', '.join(_MARKUP_TYPES.iterkeys())))
-        rendered = _MARKUP_TYPES[value.markup_type](value.raw)
+                              ', '.join(self.markup_choices_list)))
+        if self.escape_html:
+            raw = escape(value.raw)
+        else:
+            raw = value.raw
+        rendered = self.markup_choices_dict[value.markup_type](raw)
         setattr(model_instance, _rendered_field_name(self.attname), rendered)
         return value.raw
 
@@ -210,3 +160,15 @@ class MarkupField(models.TextField):
 # register MarkupField to use the custom widget in the Admin
 from django.contrib.admin.options import FORMFIELD_FOR_DBFIELD_DEFAULTS
 FORMFIELD_FOR_DBFIELD_DEFAULTS[MarkupField] = {'widget': widgets.AdminMarkupTextareaWidget}
+
+# allow South to handle MarkupField smoothly
+try:
+    from south.modelsinspector import add_introspection_rules
+    # For a normal MarkupField, the add_rendered_field attribute is
+    # always True, which means no_rendered_field arg will always be
+    # True in a frozen MarkupField, which is what we want.
+    add_introspection_rules(rules=[
+        ( (MarkupField,), [], { 'rendered_field': ['rendered_field', {}], })
+    ], patterns=['markupfield\.fields\.MarkupField'])
+except ImportError:
+    pass
